/*
 *   kioskgui.h
 *
 *   Copyright (C) 2003,2004 Waldo Bastian <bastian@kde.org>
 *   Copyright (C) 2009 Ian Reinhart Geiser <geiseri@kde.org>
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License version 2 as
 *   published by the Free Software Foundation.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.
 */
#ifndef _KIOSKGUI_H_
#define _KIOSKGUI_H_

#include <QListWidget>

#include <KDE/KConfig>
#include <KDE/KXmlGuiWindow>
#include "ui_profileSelectionPage_ui.h"
#include "ui_mainview.h"

class ComponentSelectionPage;
class ProfilePropsPage;
class PageWidget;
class UserManagementPage;
class KioskData;
class KioskRun;
class ComponentData;
class KAction;
class KToggleAction;


class MainView : public QWidget, public Ui::MainView
{
public:
  MainView( QWidget *parent ) : QWidget( parent ) {
    setupUi( this );
  }
};

class ProfileSelectionPageUI : public QWidget, public Ui::ProfileSelectionPageUI
{
public:
  ProfileSelectionPageUI( QWidget *parent ) : QWidget( parent ) {
    setupUi( this );
  }
};

class KioskGui : public KXmlGuiWindow
{
   Q_OBJECT
public:
   enum Page { PAGE_PROFILE_SELECTION = 1,
          PAGE_COMPONENT_SELECTION = 2,
          PAGE_COMPONENT = 3,
          PAGE_PROFILE_NEW = 4,
          PAGE_PROFILE_PROPS = 5,
          PAGE_PROFILE_ASSIGN = 6,
          PAGE_LAST = PAGE_PROFILE_ASSIGN };

   KioskGui();
   ~KioskGui();

   void setupActions();
   void saveProperties(KConfigGroup &config);
   void readProperties(const KConfigGroup &config);

   void selectPage(Page page);
   void loadPage(Page page);
   bool savePage(Page page);

   void loadProfiles();

public slots:
   void updateButtons();
   void finishedPage(bool save=true);
   void discardPage();
   void slotDeleteProfile(QTreeWidgetItem *item=0);
   void slotAddProfile();
   void slotProfileProperties();
   void slotProfileSetup();
   void slotManageUsers();
   void slotConfig();
   void slotComponentSelection();

   void uploadAllProfiles();
   void uploadCurrentProfile();

   void slotCheckEtcSkel();
   void help();

protected:
   void updateActions();
   void setSubCaption(const QString &subCaption);
   void setMainCaption( const QString &caption );

   virtual bool queryClose();

private:
   MainView *m_view;
   ProfileSelectionPageUI *m_profileSelectionPage;
   ComponentSelectionPage *m_componentSelectionPage;
   PageWidget *m_componentPage;
   UserManagementPage *m_profileAssignPage;
   ProfilePropsPage *m_profilePropsPage;

   KioskData *m_data;
   KioskRun *m_run;

   QString m_profile;
   QString m_component;
   ComponentData * m_componentData;
   KAction *m_uploadAction;
   KToggleAction *m_backgroundAction;
   QMap<Page,int> m_pageMapping;
   Page m_page;
};

#endif
